package de.intevation.artifacts;

import java.io.IOException;
import java.io.OutputStream;

import org.w3c.dom.Document;

/**
 * Interface of an artifact managing database.
 *
 * @author <a href="mailto:sascha.teichmann@intevation.de">Sascha L. Teichmann</a>
 */
public interface ArtifactDatabase
{
    public interface DeferredOutput {

        void write(OutputStream output) throws IOException;

    } // interface DeferredOut

    /**
     * List of artifact factories names accessible through the database.
     * @return pairs of names and descriptions of the factories.
     */
    String [][] artifactFactoryNamesAndDescriptions();

    ArtifactFactory getInternalArtifactFactory(String factoryName);

    Document createArtifactWithFactory(String factory, 
                                       CallMeta callMeta,
                                       Document data)
        throws ArtifactDatabaseException;

    Document describe(String artifact, Document data, CallMeta callMeta)
        throws ArtifactDatabaseException;

    Document advance(String artifact, Document target, CallMeta callMeta)
        throws ArtifactDatabaseException;

    Document feed(String artifact, Document data, CallMeta callMeta)
        throws ArtifactDatabaseException;

    DeferredOutput out(String artifact, Document format, CallMeta callMeta)
        throws ArtifactDatabaseException;

    Document exportArtifact(String artifact, CallMeta callMeta)
        throws ArtifactDatabaseException;

    Document importArtifact(Document data, CallMeta callMeta)
        throws ArtifactDatabaseException;

    String [][] serviceNamesAndDescriptions();

    Document process(String service, Document input, CallMeta callMeta)
        throws ArtifactDatabaseException;
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
